@@ -1,6 +1,6 @@
 /* Autokill list functions.
  *
- * IRC Services is copyright (c) 1996-2001 Andrew Church.
+ * IRC Services is copyright (c) 1996-2002 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts copyright (c) 1999-2000 Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -239,7 +239,13 @@ static void send_akill(const Akill *akill)
     }
     *host++ = 0;
 
-#ifdef IRC_BAHAMUT
+#if defined(IRC_UNREAL)
+    send_cmd(ServerName,
+	    "TKL + G %s %s %s %ld %ld :%s",
+	    username, host, akill->who ? akill->who : "<unknown>",
+	    akill->expires, akill->time,
+	    StaticAkillReason ? StaticAkillReason : akill->reason);
+#elif defined(IRC_BAHAMUT)
     send_cmd(ServerName,
 	    "AKILL %s %s %ld %s %ld :%s",
 	    host, username,
@@ -270,12 +276,16 @@ static void send_akill(const Akill *akill)
 
 static void cancel_akill(char *mask)
 {
-#ifdef IRC_DALNET
+#if defined(IRC_UNREAL) || defined(IRC_DALNET)
     char *s = strchr(mask, '@');
     if (s) {
 	*s++ = 0;
 	strlower(s);
+# if defined(IRC_UNREAL)
+	send_cmd(ServerName, "TKL - G %s %s %s", mask, s, ServerName);
+# elif defined(IRC_DALNET)
 	send_cmd(ServerName, "RAKILL %s %s", s, mask);
+# endif
     }
 #endif
 }
@@ -299,13 +309,29 @@ int check_akill(const char *nick, const char *username, const char *host)
     snprintf(buf, sizeof(buf), "%s@%s", username, host);
     for (i = 0; i < nakill; i++) {
 	if (match_wild_nocase(akills[i].mask, buf)) {
-	    /* Don't use kill_user(); that's for people who have already
-	     * signed on.  This is called before the User structure is
-	     * created. */
-	    send_cmd(s_OperServ, "KILL %s :%s (%s)", nick, s_OperServ,
-		     StaticAkillReason ? StaticAkillReason : akills[i].reason);
-	    send_akill(&akills[i]);
-	    return 1;
+	    if (akills[i].expires && akills[i].expires <= time(NULL)) {
+		/* Presumably not needed, since the user was able to
+		 * connect in the first place, but just in case... */
+		if (WallAkillExpire)
+		    wallops(s_OperServ, "AKILL on %s has expired",
+			    akills[i].mask);
+		cancel_akill(akills[i].mask);
+		free(akills[i].mask);
+		free(akills[i].reason);
+		nakill--;
+		if (i < nakill)
+		    memmove(akills+i, akills+i+1, sizeof(*akills)*(nakill-i));
+		i--;
+	    } else {
+		/* Don't use kill_user(); that's for people who have
+		 * already signed on.  This is called before the User
+		 * structure is created. */
+		send_cmd(s_OperServ, "KILL %s :%s (%s)", nick, s_OperServ,
+			 StaticAkillReason ? StaticAkillReason
+					   : akills[i].reason);
+		send_akill(&akills[i]);
+		return 1;
+	    }
 	}
     }
     return 0;
@@ -325,15 +351,58 @@ void expire_akills(void)
 	    continue;
 	if (WallAkillExpire)
 	    wallops(s_OperServ, "AKILL on %s has expired", akills[i].mask);
+	cancel_akill(akills[i].mask);
 	free(akills[i].mask);
 	free(akills[i].reason);
 	nakill--;
 	if (i < nakill)
-	    memmove(akills+i, akills+i+1, sizeof(*akills) * (nakill-i));
+	    memmove(akills+i, akills+i+1, sizeof(*akills)*(nakill-i));
 	i--;
     }
 }
 
+/*************************************************************************/
+
+#ifdef IRC_UNREAL
+
+void m_tkl(char *source, int ac, char **av)
+{
+    char buf[BUFSIZE];
+    int i;
+
+    if (noakill || ac < 4 || (*av[0] != '+' && *av[0] != '-') || *av[1] != 'G')
+	return;
+
+    snprintf(buf, sizeof(buf), "%s@%s", av[2], av[3]);
+    for (i = 0; i < nakill; i++) {
+	if (stricmp(akills[i].mask, buf) == 0) {
+	    if (akills[i].expires && akills[i].expires <= time(NULL)) {
+		/* This autokill has expired, clear it if necessary */
+		if (WallAkillExpire)
+		    wallops(s_OperServ, "AKILL on %s has expired",
+			    akills[i].mask);
+		if (*av[0] == '+')
+		    cancel_akill(akills[i].mask);
+		free(akills[i].mask);
+		free(akills[i].reason);
+		nakill--;
+		if (i < nakill)
+		    memmove(akills+i, akills+i+1, sizeof(*akills)*(nakill-i));
+		i--;
+	    } else if (*av[0] == '-') {
+		/* Autokill is still valid, re-send */
+		send_akill(&akills[i]);
+	    }
+	    return;
+	}
+    }
+    /* Not found; if it was a +, clear it */
+    if (*av[0] == '+')
+	cancel_akill(buf);
+}
+
+#endif
+
 /*************************************************************************/
 /************************** AKILL list editing ***************************/
 /*************************************************************************/
