@@ -1,6 +1,6 @@
 /* ChanServ functions.
  *
- * IRC Services is copyright (c) 1996-2001 Andrew Church.
+ * IRC Services is copyright (c) 1996-2002 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts copyright (c) 1999-2000 Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -700,6 +700,7 @@ static void timeout_leave(Timeout *to)
 
 int check_kick(User *user, const char *chan)
 {
+    Channel *c = findchan(chan);
     ChannelInfo *ci = cs_findchan(chan);
     AutoKick *akick;
     int i;
@@ -784,7 +785,7 @@ kick:
     }
     /* Remember that the user has not been added to our channel user list
      * yet, so we check whether the channel does not exist */
-    stay = !findchan(chan);
+    stay = (c == NULL);
     av[0] = (char *)chan;
     if (stay) {
 	send_cmd(s_ChanServ, "JOIN %s", chan);
@@ -799,6 +800,10 @@ kick:
 	mask[0] = '*';
 	mask[1] = '!';
     }
+    /* Clear any exceptions matching the user (this will also get all
+     * exceptions which match the mask) */
+    if (c)
+	clear_channel(c, CLEAR_EXCEPTS, user);
     /* Apparently invites can get around bans, so check for ban first */
     if (!chan_has_ban(chan, mask)) {
 	av[1] = "+b";
@@ -2983,6 +2988,7 @@ static void do_forbid(User *u)
 	log("%s: %s set FORBID for channel %s", s_ChanServ, u->nick,
 		ci->name);
 	ci->flags |= CI_VERBOTEN;
+	ci->time_registered = time(NULL);
 	notice_lang(s_ChanServ, u, CHAN_FORBID_SUCCEEDED, chan);
 	c = findchan(chan);
 	if (c)
