@@ -3,10 +3,6 @@
 
 /*************************************************************************/
 
-#ifndef bzero
-# define bzero(X,Y) memset(X,0,Y)
-#endif
-
 #ifdef MEMCHECKS
 # ifdef SHOWALLOCS
 extern int showallocs;
