@@ -2,7 +2,7 @@
  * by Andrew Kempe (TheShadow)
  *     E-mail: <theshadow@shadowfire.org>
  *
- * IRC Services is copyright (c) 1996-2001 Andrew Church.
+ * IRC Services is copyright (c) 1996-2002 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts copyright (c) 1999-2000 Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -386,6 +386,7 @@ void load_ss_dbase()
     int16 n;
     int32 tmp32;
     ServerStats *serverstats, *last;
+    time_t def_quit = time(NULL)-1;  /* avoid join>quit staying true on load */
 
     if (!(f = open_db(s_StatServ, StatDBName, "r")))
         return;
@@ -411,7 +412,7 @@ void load_ss_dbase()
 	    SAFE(read_int32(&tmp32, f));
 	    serverstats->t_join = tmp32;
 	    SAFE(read_int32(&tmp32, f));
-	    serverstats->t_quit = tmp32;
+	    serverstats->t_quit = def_quit;
 	    SAFE(read_string(&serverstats->quit_message, f));
         }
 	if (serverstats)
